@@ -3,6 +3,8 @@
 #include <pl/core/ast/ast_node.hpp>
 #include <pl/core/ast/ast_node_attribute.hpp>
 
+#include <pl/patterns/pattern_enum.hpp>
+
 #include <map>
 
 namespace pl::core::ast {
@@ -18,10 +20,15 @@ namespace pl::core::ast {
         }
 
         [[nodiscard]] std::vector<std::shared_ptr<ptrn::Pattern>> createPatterns(Evaluator *evaluator) const override;
+        std::unique_ptr<ASTNode> evaluate(Evaluator *evaluator) const override;
 
         [[nodiscard]] const std::map<std::string, std::pair<std::unique_ptr<ASTNode>, std::unique_ptr<ASTNode>>> &getEntries() const { return this->m_entries; }
+        [[nodiscard]] const ptrn::PatternEnum::EnumValue& getEnumValue(Evaluator *evaluator, const std::string &name) const;
+        [[nodiscard]] const std::map<std::string, ptrn::PatternEnum::EnumValue>& getEnumValues(Evaluator *evaluator) const;
         void addEntry(const std::string &name, std::unique_ptr<ASTNode> &&minExpr, std::unique_ptr<ASTNode> &&maxExpr) {
             this->m_entries[name] = { std::move(minExpr), std::move(maxExpr) };
+
+            this->m_cachedEnumValues.clear();
         }
 
         [[nodiscard]] const std::unique_ptr<ASTNode> &getUnderlyingType() { return this->m_underlyingType; }
@@ -29,6 +36,8 @@ namespace pl::core::ast {
     private:
         std::map<std::string, std::pair<std::unique_ptr<ASTNode>, std::unique_ptr<ASTNode>>> m_entries;
         std::unique_ptr<ASTNode> m_underlyingType;
+
+        mutable std::map<std::string, ptrn::PatternEnum::EnumValue> m_cachedEnumValues;
     };
 
 }
\ No newline at end of file
