@@ -230,7 +230,7 @@ namespace pl::ptrn {
             return Pattern::getTypeName();
         }
 
-        void setEnumValues(const std::vector<PatternEnum::EnumValue> &enumValues) {
+        void setEnumValues(const std::map<std::string, PatternEnum::EnumValue> &enumValues) {
             this->m_enumValues = enumValues;
         }
 
@@ -243,14 +243,9 @@ namespace pl::ptrn {
                 return false;
 
             auto &otherEnum = *static_cast<const PatternBitfieldFieldEnum *>(&other);
-            if (this->m_enumValues.size() != otherEnum.m_enumValues.size())
+            if (this->m_enumValues != otherEnum.m_enumValues)
                 return false;
 
-            for (u64 i = 0; i < this->m_enumValues.size(); i++) {
-                if (this->m_enumValues[i] != otherEnum.m_enumValues[i])
-                    return false;
-            }
-
             return true;
         }
 
@@ -270,7 +265,7 @@ namespace pl::ptrn {
         }
 
     private:
-        std::vector<PatternEnum::EnumValue> m_enumValues;
+        std::map<std::string, PatternEnum::EnumValue> m_enumValues;
     };
 
     class PatternBitfieldArray : public PatternBitfieldMember,
@@ -433,8 +428,8 @@ namespace pl::ptrn {
             }
         }
 
-        void setEntries(std::vector<std::shared_ptr<Pattern>> &&entries) override {
-            this->m_entries = std::move(entries);
+        void setEntries(const std::vector<std::shared_ptr<Pattern>> &entries) override {
+            this->m_entries = entries;
 
             for (auto &entry : this->m_entries) {
                 if (!entry->hasOverriddenColor())
@@ -779,8 +774,8 @@ namespace pl::ptrn {
             return this->m_fields;
         }
 
-        void setEntries(std::vector<std::shared_ptr<Pattern>> &&entries) override {
-            this->m_fields = std::move(entries);
+        void setEntries(const std::vector<std::shared_ptr<Pattern>> &entries) override {
+            this->m_fields = entries;
         }
 
         void setOffset(u64 offset) override {
