@@ -81,17 +81,17 @@ namespace pl::core::ast {
             auto type = this->m_type->evaluate(evaluator);
 
             std::shared_ptr<ptrn::Pattern> pattern;
-            if (dynamic_cast<ASTNodeBuiltinType *>(type.get()) != nullptr)
+            if (auto builtinType = dynamic_cast<ASTNodeBuiltinType *>(type.get()); builtinType != nullptr && builtinType->getType() != Token::ValueType::CustomType)
                 pattern = createStaticArray(evaluator);
-            else if (auto attributable = dynamic_cast<Attributable *>(type.get())) {
-                bool isStaticType = attributable->hasAttribute("static", false);
+            else {
+                bool isStaticType = false;
+                if (auto attributable = dynamic_cast<Attributable *>(type.get()))
+                    isStaticType = attributable->hasAttribute("static", false);
 
                 if (isStaticType)
                     pattern = createStaticArray(evaluator);
                 else
                     pattern = createDynamicArray(evaluator);
-            } else {
-                err::E0001.throwError("Invalid type used in array variable declaration.", { }, this->getLocation());
             }
 
             pattern->setSection(evaluator->getSectionId());
@@ -272,7 +272,7 @@ namespace pl::core::ast {
 
         auto addEntries = [&](std::vector<std::shared_ptr<ptrn::Pattern>> &&patterns) {
             for (auto &pattern : patterns) {
-                pattern->setVariableName(fmt::format("[{}]", entryIndex));
+                pattern->setArrayIndex(entryIndex);
                 pattern->setEndian(arrayPattern->getEndian());
                 if (pattern->getSection() == ptrn::Pattern::MainSectionId)
                     pattern->setSection(arrayPattern->getSection());
@@ -420,7 +420,7 @@ namespace pl::core::ast {
         if (arrayPattern->getEntryCount() > 0)
             arrayPattern->setTypeName(arrayPattern->getEntry(0)->getTypeName());
 
-        arrayPattern->setEntries(std::move(entries));
+        arrayPattern->setEntries(entries);
         arrayPattern->setSize(size);
 
         return arrayPattern;
