@@ -40,6 +40,19 @@ namespace pl::core::ast {
             pattern = std::make_unique<ptrn::PatternPadding>(evaluator, offset, 1, getLocation().line);
         else if (this->m_type == Token::ValueType::String)
             pattern = std::make_unique<ptrn::PatternString>(evaluator, offset, 0, getLocation().line);
+        else if (this->m_type == Token::ValueType::CustomType) {
+            std::vector<Token::Literal> params;
+
+            const auto &templateParams = evaluator->getTemplateParameters();
+            for (u32 i = templateParams.size() - m_parameterCount.min; i < templateParams.size(); i += 1) {
+                params.emplace_back(templateParams[i]->getValue());
+            }
+
+            pattern = m_customTypeCallback(evaluator, params);
+
+            // The size of the custom built-in type is only known now so we need to increment the cursor here manually again
+            std::ignore = evaluator->getReadOffsetAndIncrement(pattern->getSize());
+        }
         else if (this->m_type == Token::ValueType::Auto)
             return { };
         else
