@@ -9,12 +9,13 @@
 #include <pl/core/ast/ast_node_function_definition.hpp>
 #include <pl/core/ast/ast_node_compound_statement.hpp>
 #include <pl/core/ast/ast_node_control_flow_statement.hpp>
+#include <pl/core/ast/ast_node_lvalue_assignment.hpp>
+#include <pl/core/ast/ast_node_literal.hpp>
+#include <pl/core/ast/ast_node_builtin_type.hpp>
 
 #include <pl/patterns/pattern_unsigned.hpp>
 #include <pl/patterns/pattern_signed.hpp>
 #include <pl/patterns/pattern_enum.hpp>
-#include <pl/patterns/pattern_struct.hpp>
-#include <pl/patterns/pattern_union.hpp>
 #include <pl/patterns/pattern_float.hpp>
 #include <pl/patterns/pattern_boolean.hpp>
 #include <pl/patterns/pattern_character.hpp>
@@ -25,6 +26,7 @@
 
 #include <exception>
 #include <utility>
+#include "wolv/utils/string.hpp"
 
 namespace pl::core {
 
@@ -224,7 +226,7 @@ namespace pl::core {
 
                 entries.push_back(std::move(entryPattern));
             }
-            pattern->setEntries(std::move(entries));
+            pattern->setEntries(entries);
             pattern->setSection(section);
 
         } else if (section == ptrn::Pattern::HeapSectionId) {
@@ -240,7 +242,7 @@ namespace pl::core {
 
                 entries.push_back(std::move(entryPattern));
             }
-            pattern->setEntries(std::move(entries));
+            pattern->setEntries(entries);
             pattern->setLocal(true);
         } else {
             typePattern->setSection(section);
@@ -250,7 +252,7 @@ namespace pl::core {
                 entryPattern->setOffset(entryPattern->getSize() * i);
                 entries.push_back(std::move(entryPattern));
             }
-            pattern->setEntries(std::move(entries));
+            pattern->setEntries(entries);
             pattern->setSection(section);
         }
 
@@ -263,12 +265,59 @@ namespace pl::core {
         variables.push_back(std::unique_ptr<ptrn::Pattern>(pattern));
     }
 
-    static std::optional<std::string> findTypeName(const ast::ASTNodeTypeDecl *type) {
-
+    std::optional<std::string> Evaluator::findTypeName(const ast::ASTNodeTypeDecl *type) {
         const ast::ASTNodeTypeDecl *typeDecl = type;
         while (true) {
-            if (auto name = typeDecl->getName(); !name.empty())
-                return name;
+            if (auto name = typeDecl->getName(); !name.empty()) {
+                if (const auto &templateParams = typeDecl->getTemplateParameters(); templateParams.empty()) {
+                    return name;
+                } else {
+                    std::string templateTypeString;
+                    for (const auto &templateParameter : templateParams) {
+                        if (auto lvalue = dynamic_cast<ast::ASTNodeLValueAssignment *>(templateParameter.get())) {
+                            if (!lvalue->getRValue())
+                                err::E0003.throwError(fmt::format("No value set for non-type template parameter {}. This is a bug.", lvalue->getLValueName()), {}, type->getLocation());
+                            auto valueNode = lvalue->getRValue()->evaluate(this);
+                            if (auto literal = dynamic_cast<ast::ASTNodeLiteral*>(valueNode.get()); literal != nullptr) {
+                                const auto &value = literal->getValue();
+
+                                if (value.isString()) {
+                                    auto string = value.toString();
+                                    if (string.size() > 32)
+                                        string = "...";
+                                    templateTypeString += fmt::format("\"{}\", ", hlp::encodeByteString({ string.begin(), string.end() }));
+                                } else if (value.isPattern()) {
+                                    templateTypeString += fmt::format("{}{{ }}, ", value.toPattern()->getTypeName());
+                                } else {
+                                    templateTypeString += fmt::format("{}, ", value.toString(true));
+                                }
+                            } else {
+                                err::E0003.throwError(fmt::format("Template parameter {} is not a literal. This is a bug.", lvalue->getLValueName()), {}, type->getLocation());
+                            }
+                        } else if (const auto *typeNode = dynamic_cast<ast::ASTNodeTypeDecl*>(templateParameter.get())) {
+                            const auto *node = typeNode->getType().get();
+                            while (node != nullptr) {
+                                if (const auto *innerNode = dynamic_cast<const ast::ASTNodeTypeDecl*>(node)) {
+                                    if (const auto &innerNodeName = innerNode->getName(); !innerNodeName.empty()) {
+                                        templateTypeString += fmt::format("{}, ", innerNodeName);
+                                        break;
+                                    }
+                                    node = innerNode->getType().get();
+                                }
+                                if (const auto *innerNode = dynamic_cast<const ast::ASTNodeBuiltinType*>(node)) {
+                                    templateTypeString += fmt::format("{}, ", Token::getTypeName(innerNode->getType()));
+
+                                    break;
+                                }
+                            }
+                        }
+                    }
+
+                    templateTypeString = templateTypeString.substr(0, templateTypeString.size() - 2);
+
+                    return fmt::format("{}<{}>", name, templateTypeString);
+                }
+            }
             else if (auto innerType = dynamic_cast<ast::ASTNodeTypeDecl*>(typeDecl->getType().get()); innerType != nullptr)
                 typeDecl = innerType;
             else
@@ -573,7 +622,7 @@ namespace pl::core {
         this->setVariable(pattern, variableValue);
     }
 
-    void Evaluator::changePatternType(std::shared_ptr<ptrn::Pattern> &pattern, std::shared_ptr<ptrn::Pattern> &&newPattern) {
+    void Evaluator::changePatternType(std::shared_ptr<ptrn::Pattern> &pattern, std::shared_ptr<ptrn::Pattern> &&newPattern) const {
         if (dynamic_cast<ptrn::PatternPadding*>(pattern.get()) == nullptr)
             return;
 
@@ -755,7 +804,7 @@ namespace pl::core {
 
         const auto &heap = this->getHeap();
 
-        this->m_scopes.emplace_back(std::make_unique<Scope>(parent, &scope, std::nullopt, heap.size()));
+        this->m_scopes.emplace_back(std::make_unique<Scope>(parent, &scope, heap.size()));
 
         if (this->isDebugModeEnabled())
             this->getConsole().log(LogConsole::Level::Debug, fmt::format("Entering new scope #{}. Parent: '{}', Heap Size: {}.", this->m_scopes.size(), parent == nullptr ? "None" : parent->getVariableName(), heap.size()));
@@ -800,10 +849,10 @@ namespace pl::core {
 
         if (sectionId == ptrn::Pattern::MainSectionId) [[likely]] {
             if (!write) [[likely]] {
-                this->m_readerFunction(address, reinterpret_cast<u8*>(buffer), size);
+                this->m_readerFunction(address, static_cast<u8*>(buffer), size);
             } else {
                 if (address < this->m_dataBaseAddress + this->m_dataSize)
-                    this->m_writerFunction(address, reinterpret_cast<u8*>(buffer), size);
+                    this->m_writerFunction(address, static_cast<u8*>(buffer), size);
             }
         } else if (sectionId == ptrn::Pattern::HeapSectionId) {
             auto &heap = this->getHeap();
@@ -906,6 +955,13 @@ namespace pl::core {
             err::E0011.throwError(fmt::format("Tried accessing a non-existing section with id {}.", id));
     }
 
+    u64 Evaluator::getSectionSize(u64 id) {
+        if (id == ptrn::Pattern::MainSectionId)
+            return this->getDataSize();
+        else
+            return this->getSection(id).size();
+    }
+
     const std::map<u64, api::Section> &Evaluator::getSections() const {
         return this->m_sections;
     }
@@ -932,11 +988,14 @@ namespace pl::core {
         this->m_heap.clear();
         this->m_patternLocalStorage.clear();
         this->m_templateParameters.clear();
+        this->m_stringPool.clear();
 
         this->m_mainResult.reset();
-        this->m_colorIndex = 0;
         this->m_aborted = false;
         this->m_evaluated = false;
+        this->m_attributedPatterns.clear();
+
+        this->setPatternColorPalette(DefaultPatternColorPalette);
 
         if (this->m_allowDangerousFunctions == DangerousFunctionPermission::Deny)
             this->m_allowDangerousFunctions = DangerousFunctionPermission::Ask;
@@ -958,6 +1017,18 @@ namespace pl::core {
         if (this->isDebugModeEnabled())
             this->m_console.log(LogConsole::Level::Debug, fmt::format("Base Pattern size: 0x{:02X} bytes", sizeof(ptrn::Pattern)));
 
+        this->m_sourceLineLength.clear();
+        for (auto &topLevelNode : ast) {
+            if (topLevelNode->getLocation().source->mainSource) {
+                std::vector<std::string> sourceLines = wolv::util::splitString(topLevelNode->getLocation().source->content, "\n");
+                for (const auto &sourceLine : sourceLines) {
+                    this->m_sourceLineLength.push_back(sourceLine.size());
+                }
+                break;
+            }
+        }
+        this->m_lastPauseLine = std::nullopt;
+
         try {
             this->setCurrentControlFlowStatement(ControlFlowStatement::None);
             this->pushScope(nullptr, this->m_patterns);
@@ -1098,17 +1169,19 @@ namespace pl::core {
         if (node != nullptr) {
             auto rawLine = node->getLocation().line;
             const auto line = rawLine + (rawLine == 0);
-            if (evaluator->m_shouldPauseNextLine && evaluator->m_lastPauseLine != line) {
-                evaluator->m_shouldPauseNextLine = false;
-                evaluator->m_lastPauseLine = line;
-                evaluator->m_breakpointHitCallback();
-            } else if (evaluator->m_breakpoints.contains(line)) {
-                if (evaluator->m_lastPauseLine != line) {
-                    evaluator->m_lastPauseLine = line;
-                    evaluator->m_breakpointHitCallback();
+            auto rawColumn = node->getLocation().column;
+            const auto column = rawColumn + (rawColumn == 0);
+            if (const auto source = node->getLocation().source; source != nullptr && source->mainSource) {
+                if (evaluator->m_lastPauseLine != line && column < evaluator->m_sourceLineLength[line - 1]) {
+                    if (evaluator->m_shouldPauseNextLine || evaluator->m_breakpoints.contains(line)) {
+                        if (evaluator->m_shouldPauseNextLine)
+                            evaluator->m_shouldPauseNextLine = false;
+                        evaluator->m_lastPauseLine = line;
+                        evaluator->m_breakpointHitCallback();
+                    } else if (!evaluator->m_breakpoints.contains(line))
+                        evaluator->m_lastPauseLine = std::nullopt;
                 }
             }
-
             evaluator->m_callStack.push_back(node->clone());
         }
     }
@@ -1118,7 +1191,7 @@ namespace pl::core {
             return;
 
         // Don't pop scopes if an exception is currently being thrown so we can generate
-        // a stack tace
+        // a stack trace
         if (std::uncaught_exceptions() > 0)
             return;
 
@@ -1129,23 +1202,24 @@ namespace pl::core {
         return { this, node };
     }
 
-    void Evaluator::addBreakpoint(u64 line) { this->m_breakpoints.insert(line); }
-    void Evaluator::removeBreakpoint(u64 line) { this->m_breakpoints.erase(line); }
+    void Evaluator::addBreakpoint(u32 line) { this->m_breakpoints.insert(line); }
+    void Evaluator::removeBreakpoint(u32 line) { this->m_breakpoints.erase(line); }
     void Evaluator::clearBreakpoints() { this->m_breakpoints.clear(); }
     void Evaluator::setBreakpointHitCallback(const std::function<void()> &callback) { this->m_breakpointHitCallback = callback; }
-    const std::unordered_set<int> &Evaluator::getBreakpoints() const { return this->m_breakpoints; }
+    const std::unordered_set<u32> &Evaluator::getBreakpoints() const { return this->m_breakpoints; }
+    void Evaluator::setBreakpoints(const std::unordered_set<u32> &breakpoints) { m_breakpoints = breakpoints; }
     void Evaluator::pauseNextLine() { this->m_shouldPauseNextLine = true; }
 
     std::optional<u32> Evaluator::getPauseLine() const {
         return this->m_lastPauseLine;
     }
 
-    void Evaluator::patternCreated(const ptrn::Pattern *pattern) {
+    void Evaluator::patternCreated(ptrn::Pattern *pattern) {
         this->m_lastPatternAddress = pattern->getOffset();
 
-        if (this->m_currPatternCount > this->m_patternLimit && !this->m_evaluated)
+        if (this->m_patternLimit > 0 && this->m_currPatternCount > this->m_patternLimit && !this->m_evaluated)
             err::E0007.throwError(fmt::format("Pattern count exceeded set limit of '{}'.", this->getPatternLimit()), "If this is intended, try increasing the limit using '#pragma pattern_limit <new_limit>'.");
-        this->m_currPatternCount++;
+        this->m_currPatternCount += 1;
 
         // Make sure we don't throw an error if we're already in an error state
         if (std::uncaught_exceptions() != 0)
@@ -1162,13 +1236,20 @@ namespace pl::core {
         }
     }
 
-    void Evaluator::patternDestroyed(const ptrn::Pattern *pattern) {
-        this->m_currPatternCount--;
+    void Evaluator::patternDestroyed(ptrn::Pattern *pattern) {
+        this->m_currPatternCount -= 1;
 
         // Make sure we don't throw an error if we're already in an error state
         if (std::uncaught_exceptions() != 0)
             return;
 
+        const auto &attributes = pattern->getAttributes();
+        if (attributes != nullptr) {
+            for (const auto &[attribute, args] : *attributes) {
+                this->removeAttributedPattern(attribute, pattern);
+            }
+        }
+
         if (pattern->isPatternLocal()) {
             if (auto it = this->m_patternLocalStorage.find(pattern->getHeapAddress()); it != this->m_patternLocalStorage.end()) {
                 auto &[key, data] = *it;
