@@ -13,10 +13,9 @@ using Result = pl::hlp::CompileResult<ParserManager::ParsedData>;
 Result ParserManager::parse(api::Source *source, const std::string &namespacePrefix) {
     OnceIncludePair key = { source, namespacePrefix };
 
-    if (m_onceIncluded.contains( key )) {
+    if (m_onceIncluded.contains(key)) {
         const auto& types = m_parsedTypes[key];
-        if (!types.empty())
-            return Result::good({ {}, types });
+        return Result::good({ {}, types }); // Even if types is empty we still need to return now
     }
 
     Parser parser;
