@@ -2,10 +2,10 @@
 
 #include <hex.hpp>
 #include <hex/api/localization_manager.hpp>
+#include <hex/api/shortcut_manager.hpp>
 #include <hex/helpers/concepts.hpp>
 
 #include <functional>
-#include <map>
 #include <mutex>
 #include <span>
 #include <string>
@@ -24,7 +24,6 @@ enum ImGuiCustomCol : int;
 namespace hex {
 
     class View;
-    class Shortcut;
     class Task;
 
     namespace dp {
@@ -381,7 +380,7 @@ namespace hex {
                 };
 
                 using DisplayCallback = std::function<std::string(std::string)>;
-                using ExecuteCallback = std::function<void(std::string)>;
+                using ExecuteCallback = std::function<std::optional<std::string>(std::string)>;
                 using QueryCallback   = std::function<std::vector<QueryResult>(std::string)>;
 
                 struct Entry {
@@ -417,7 +416,7 @@ namespace hex {
                 const std::string &command,
                 const UnlocalizedString &unlocalizedDescription,
                 const impl::DisplayCallback &displayCallback,
-                const impl::ExecuteCallback &executeCallback = [](auto) {});
+                const impl::ExecuteCallback &executeCallback = [](auto) { return std::nullopt; });
 
             /**
              * @brief Adds a new command handler to the command palette
@@ -438,7 +437,7 @@ namespace hex {
 
             namespace impl {
 
-                using VisualizerFunctionCallback = std::function<void(pl::ptrn::Pattern&, pl::ptrn::IIterable&, bool, std::span<const pl::core::Token::Literal>)>;
+                using VisualizerFunctionCallback = std::function<void(pl::ptrn::Pattern&, bool, std::span<const pl::core::Token::Literal>)>;
 
                 struct FunctionDefinition {
                     pl::api::Namespace ns;
@@ -450,6 +449,14 @@ namespace hex {
                     bool dangerous;
                 };
 
+                struct TypeDefinition {
+                    pl::api::Namespace ns;
+                    std::string name;
+
+                    pl::api::FunctionParameterCount parameterCount;
+                    pl::api::TypeCallback callback;
+                };
+
                 struct Visualizer {
                     pl::api::FunctionParameterCount parameterCount;
                     VisualizerFunctionCallback callback;
@@ -459,6 +466,7 @@ namespace hex {
                 const std::map<std::string, Visualizer>& getInlineVisualizers();
                 const std::map<std::string, pl::api::PragmaHandler>& getPragmas();
                 const std::vector<FunctionDefinition>& getFunctions();
+                const std::vector<TypeDefinition>& getTypes();
 
             }
 
@@ -517,6 +525,20 @@ namespace hex {
                 const pl::api::FunctionCallback &func
             );
 
+            /**
+             * @brief Adds a new type to the pattern language
+             * @param ns The namespace of the type
+             * @param name The name of the type
+             * @param parameterCount The amount of non-type template parameters the type takes
+             * @param func The type callback
+             */
+            void addType(
+                const pl::api::Namespace &ns,
+                const std::string &name,
+                pl::api::FunctionParameterCount parameterCount,
+                const pl::api::TypeCallback &func
+            );
+
             /**
              * @brief Adds a new visualizer to the pattern language
              * @note Visualizers are extensions to the [[hex::visualize]] attribute, used to visualize data
@@ -748,7 +770,7 @@ namespace hex {
                 struct MenuItem {
                     std::vector<UnlocalizedString> unlocalizedNames;
                     Icon icon;
-                    std::unique_ptr<Shortcut> shortcut;
+                    Shortcut shortcut;
                     View *view;
                     MenuCallback callback;
                     EnabledCallback enabledCallback;
@@ -1273,7 +1295,7 @@ namespace hex {
                 void stopServices();
             }
 
-            void registerService(const UnlocalizedString &unlocalizedName, const impl::Callback &callback);
+            void registerService(const UnlocalizedString &unlocalizedString, const impl::Callback &callback);
         }
 
         /* Network Communication Interface Registry. Allows adding new communication interface endpoints */
