@@ -1,7 +1,7 @@
 #pragma once
 
 #include <string_view>
-#include <fmt/format.h>
+#include <fmt/core.h>
 #include <fmt/ranges.h>
 
 namespace hex {
